/**
 * Copyright (c) 2006, 2008 Oracle and/or its affiliates. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Mohamad Raja - Initial implementation
 */
package org.eclipse.higgins.idas.cp.google.test;

import java.io.File;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.Iterator;

import junit.framework.TestCase;

import org.eclipse.higgins.idas.api.IAttribute;
import org.eclipse.higgins.idas.api.IComplexAttrValue;
import org.eclipse.higgins.idas.api.IContext;
import org.eclipse.higgins.idas.api.IContextId;
import org.eclipse.higgins.idas.api.IEntity;
import org.eclipse.higgins.idas.api.IFilter;
import org.eclipse.higgins.idas.api.IFilterAttributeAssertion;
import org.eclipse.higgins.idas.api.ISimpleAttrValue;
import org.eclipse.higgins.idas.api.ITypedValue;
import org.eclipse.higgins.idas.api.IdASException;
import org.eclipse.higgins.idas.common.AuthNNamePasswordMaterials;
import org.eclipse.higgins.idas.registry.IdASRegistry;
import org.eclipse.higgins.idas.registry.contextid.ContextIdFactory;
import org.eclipse.higgins.idas.registry.discovery.FileDiscovery;
import org.eclipse.higgins.idas.spi.BasicValueInt;

public class GDataCPTest extends TestCase
{
    /** TODO Convert to JUnit test case **/
    public static void main(String[] args) throws Exception
    {
        IdASRegistry registry = IdASRegistry.getInstance();
        IContextId contextId = ContextIdFactory.fromString("GDataContext.xrds");
        registry.setDiscovery(new FileDiscovery(new File("GDataContext.xrds")));
        System.out.println("List: " + Arrays.toString(contextId.getTypes()));

        IContext context = registry.createContext("GDataContext.xrds");
        context.open(new AuthNNamePasswordMaterials(
            context, "mohamad.raja.gani@gmail.com", "******"));

        createContact(context, "New contact");
        listContacts(context);
        queryContacts(context);
    }

    private static IEntity createContact(IContext context, String entityID)
        throws IdASException, URISyntaxException
    {
        IEntity entity = context.addEntity(new URI("contact"), entityID);
        IAttribute attr = entity.addAttribute(new URI("title"));
        attr.addSimpleValue(ISimpleAttrValue.STRING_TYPE_URI, "Mohamad Raja");

        attr = entity.addAttribute(new URI("email"));
        IComplexAttrValue attrValue = 
            attr.addComplexValue(ITypedValue.ANYURI_TYPE_URI);
        IAttribute compAttr = attrValue.addAttribute(new URI("address"));
        compAttr.addSimpleValue(ISimpleAttrValue.STRING_TYPE_URI, 
            "mohamad.raja@oracle.com");

        compAttr = attrValue.addAttribute(new URI("rel"));
        compAttr.addSimpleValue(ISimpleAttrValue.STRING_TYPE_URI, 
            "http://schemas.google.com/g/2005#work");

        context.applyUpdates();
        System.out.println("Contact added successfully: " + entity);
        return entity;
    }

    private static void listContacts(IContext context) throws IdASException
    {
        System.out.println("Enter List contacts");
        Iterator entityIter = context.getEntities(null);
        while (entityIter.hasNext())
            System.out.println("\nSearch Result: " + entityIter.next());
    }

    private static void queryContacts(IContext context) 
        throws IdASException, URISyntaxException
    {
        System.out.println("Enter Query contacts");
        IFilter filter = context.buildFilter();
        filter.setOperator(IFilter.OP_AND);

        IFilter nFilter = context.buildFilter();
        IFilterAttributeAssertion attrAssertion = context.buildAttributeAssertion();
        attrAssertion.setID(new URI("maxResults"));
        attrAssertion.setComparator(IFilterAttributeAssertion.COMP_ATTR_EQ);
        BasicValueInt assertionValue = new BasicValueInt(10, null);
        attrAssertion.setAssertionValue(assertionValue);
        nFilter.setAssertion(attrAssertion);
        filter.addFilter(nFilter);

        nFilter = context.buildFilter();
        attrAssertion = context.buildAttributeAssertion();
        attrAssertion.setID(new URI("startIndex"));
        attrAssertion.setComparator(IFilterAttributeAssertion.COMP_ATTR_EQ);
        assertionValue = new BasicValueInt(1, null);
        attrAssertion.setAssertionValue(assertionValue);
        nFilter.setAssertion(attrAssertion);
        filter.addFilter(nFilter);

        Iterator entityIter = context.getEntities(filter);
        while (entityIter.hasNext())
            System.out.println("\nSearch Result: " + entityIter.next());
    }
}
